@@ -6,6 +6,7 @@ import torch
 from torch import nn, einsum
 from torch.autograd import grad as torch_grad
 import torch.nn.functional as F
+from torch.nn.utils.rnn import pad_sequence
 
 from einops import rearrange, repeat
 
@@ -27,6 +28,9 @@ def default(val, d):
 def ceil_div(numer, denom):
     return (numer + denom - 1) // denom
 
+def remainder_needed_until_multiple(n, mult):
+    return (ceil_div(n, mult) * mult) - n
+
 def round_down_nearest_multiple(val, mult):
     return (val // mult) * mult
 
@@ -56,6 +60,33 @@ def gradient_penalty(images, output, weight = 10):
     gradients = rearrange(gradients, 'b ... -> b (...)')
     return weight * ((gradients.norm(2, dim = 1) - 1) ** 2).mean()
 
+# classifier free guidance functions
+
+def uniform(shape, device):
+    return torch.zeros(shape, device = device).float().uniform_(0, 1)
+
+def prob_mask_like(shape, prob, device):
+    if prob == 1:
+        return torch.ones(shape, device = device, dtype = torch.bool)
+    elif prob == 0:
+        return torch.zeros(shape, device = device, dtype = torch.bool)
+    else:
+        return torch.zeros(shape, device = device).float().uniform_(0, 1) < prob
+
+# removing unique consecutives in the semantic token ids
+# important detail noted by @eonglints
+
+def append_eos_id(ids, eos_id):
+    b, device = ids.shape[0], ids.device
+    eos_ids = torch.ones(1, device = device).long() * eos_id
+    eos_ids = repeat(eos_ids, '1 -> b 1', b = b)
+    ids = torch.cat((ids, eos_ids), dim = -1)
+    return ids
+
+def batch_unique_consecutive(t, pad_value = 0.):
+    unique_arr = [torch.unique_consecutive(el) for el in t.unbind(dim = 0)]
+    return pad_sequence(unique_arr, batch_first = True, padding_value = pad_value)
+
 # discriminators
 
 class MultiScaleDiscriminator(nn.Module):
@@ -609,20 +640,28 @@ class SemanticTransformer(nn.Module):
         dim,
         t5_name = DEFAULT_T5_NAME,
         has_condition = False,
+        cond_drop_prob = 0.5,
         wav2vec: Optional[Union[FairseqVQWav2Vec, HubertWithKmeans]] = None,
+        unique_consecutive = True,
+        pad_id = -1,
         **kwargs
     ):
         super().__init__()
         self.has_condition = has_condition
         self.embed_text = partial(t5_encode_text, name = t5_name)
+        self.cond_drop_prob = cond_drop_prob
+
+        self.unique_consecutive = unique_consecutive
 
         self.start_token = nn.Parameter(torch.randn(dim))
 
-        self.semantic_embedding = nn.Embedding(num_semantic_tokens, dim)
+        self.semantic_embedding = nn.Embedding(num_semantic_tokens + 1, dim)
+        self.eos_id = num_semantic_tokens
+        self.pad_id = pad_id
 
         self.wav2vec = wav2vec
         self.transformer = Transformer(dim = dim, dim_context = get_encoded_dim(t5_name), cross_attend = has_condition, **kwargs)
-        self.to_logits = nn.Linear(dim, num_semantic_tokens)
+        self.to_logits = nn.Linear(dim, num_semantic_tokens + 1)
 
     def forward(
         self,
@@ -631,10 +670,24 @@ class SemanticTransformer(nn.Module):
         ids = None,
         return_loss = False,
         text = None,
-        text_embed = None
+        text_embed = None,
+        cond_drop_prob = None
     ):
         device = next(self.parameters()).device
 
+        assert exists(raw_wave) ^ exists(ids)
+
+        if not exists(ids):
+            assert exists(self.wav2vec)
+            ids = self.wav2vec(raw_wave, flatten = False)
+
+        b = ids.shape[0]
+
+        ids = append_eos_id(ids, self.eos_id)
+
+        if self.unique_consecutive:
+            ids = batch_unique_consecutive(ids)
+
         has_text = exists(text) or exists(text_embed)
         assert not (self.has_condition ^ has_text)
 
@@ -643,12 +696,12 @@ class SemanticTransformer(nn.Module):
                 text_embeds = self.embed_text(text, output_device = device)
                 text_mask = torch.any(text_embeds != 0, dim = -1)
 
-        assert exists(raw_wave) ^ exists(ids)
+        cond_drop_prob = default(cond_drop_prob, self.cond_drop_prob)
+
+        if cond_drop_prob > 0:
+            keep_mask = prob_mask_like((b,), 1 - cond_drop_prob, device = device)
+            text_mask = rearrange(keep_mask, 'b -> b 1') & text_mask
 
-        if not exists(ids):
-            assert exists(self.wav2vec)
-            ids = self.wav2vec(raw_wave, flatten = False)
-            
         if return_loss:
             labels, ids = ids.clone(), ids[:, :-1]
 
@@ -666,7 +719,8 @@ class SemanticTransformer(nn.Module):
 
         loss = F.cross_entropy(
             rearrange(logits, 'b n c -> b c n'),
-            labels
+            labels,
+            ignore_index = self.pad_id
         )
 
         return loss
@@ -681,17 +735,23 @@ class CoarseTransformer(nn.Module):
         dim,
         t5_name = DEFAULT_T5_NAME,
         has_condition = False,
+        cond_drop_prob = 0.5,
         wav2vec: Optional[Union[FairseqVQWav2Vec, HubertWithKmeans]] = None,
         **kwargs
     ):
         super().__init__()
         self.has_condition = has_condition
         self.embed_text = partial(t5_encode_text, name = t5_name)
+        self.cond_drop_prob = cond_drop_prob
 
         self.start_token = nn.Parameter(torch.randn(dim))
 
-        self.semantic_embedding = nn.Embedding(num_semantic_tokens, dim)
-        self.coarse_embedding = nn.Embedding(num_coarse_quantizers * codebook_size, dim)
+        self.semantic_eos_id = num_semantic_tokens
+        self.semantic_embedding = nn.Embedding(num_semantic_tokens + 1, dim)
+
+        self.coarse_eos_id = codebook_size
+        codebook_size_with_eos = codebook_size + 1
+        self.coarse_embedding = nn.Embedding(num_coarse_quantizers * codebook_size_with_eos, dim)
 
         self.wav2vec = wav2vec
         self.transformer = Transformer(dim = dim, dim_context = get_encoded_dim(t5_name), cross_attend = has_condition, **kwargs)
@@ -699,8 +759,8 @@ class CoarseTransformer(nn.Module):
         self.codebook_size = codebook_size
         self.num_coarse_quantizers = num_coarse_quantizers
 
-        self.to_semantic_logits = nn.Linear(dim, num_semantic_tokens)
-        self.coarse_logit_weights = nn.Parameter(torch.randn(num_coarse_quantizers, codebook_size, dim))
+        self.to_semantic_logits = nn.Linear(dim, num_semantic_tokens + 1)
+        self.coarse_logit_weights = nn.Parameter(torch.randn(num_coarse_quantizers, codebook_size_with_eos, dim))
 
     def forward(
         self,
@@ -708,7 +768,8 @@ class CoarseTransformer(nn.Module):
         semantic_token_ids,
         coarse_token_ids,
         text = None,
-        text_embed = None
+        text_embed = None,
+        cond_drop_prob = None
     ):
         b, device = semantic_token_ids.shape[0], semantic_token_ids.device
 
@@ -720,6 +781,12 @@ class CoarseTransformer(nn.Module):
                 text_embeds = self.embed_text(text, output_device = device)
                 text_mask = torch.any(text_embeds != 0, dim = -1)
 
+        cond_drop_prob = default(cond_drop_prob, self.cond_drop_prob)
+
+        if cond_drop_prob > 0:
+            keep_mask = prob_mask_like((b,), 1 - cond_drop_prob, device = device)
+            text_mask = rearrange(keep_mask, 'b -> b 1') & text_mask
+
         coarse_token_ids, semantic_token_ids = map(lambda t: rearrange(t, 'b ... -> b (...)'), (coarse_token_ids, semantic_token_ids))
 
         offsets = self.codebook_size * torch.arange(self.num_coarse_quantizers, device = device)
@@ -778,16 +845,22 @@ class FineTransformer(nn.Module):
         dim,
         t5_name = DEFAULT_T5_NAME,
         has_condition = False,
+        cond_drop_prob = 0.5,
         **kwargs
     ):
         super().__init__()
         self.has_condition = has_condition
         self.embed_text = partial(t5_encode_text, name = t5_name)
+        self.cond_drop_prob = cond_drop_prob
 
         self.start_token = nn.Parameter(torch.randn(dim))
 
-        self.coarse_embedding = nn.Embedding(num_coarse_quantizers * codebook_size, dim)
-        self.fine_embedding = nn.Embedding(num_fine_quantizers * codebook_size, dim)
+        codebook_size_with_eos = codebook_size + 1
+
+        self.coarse_embedding = nn.Embedding(num_coarse_quantizers * codebook_size_with_eos, dim)
+        self.fine_embedding = nn.Embedding(num_fine_quantizers * codebook_size_with_eos, dim)
+
+        self.eos_id = codebook_size
 
         self.transformer = Transformer(dim = dim, dim_context = get_encoded_dim(t5_name), cross_attend = has_condition, **kwargs)
 
@@ -795,18 +868,18 @@ class FineTransformer(nn.Module):
         self.num_coarse_quantizers = num_coarse_quantizers
         self.num_fine_quantizers = num_fine_quantizers
 
-        self.coarse_logit_weights = nn.Parameter(torch.randn(num_coarse_quantizers, codebook_size, dim))
-        self.fine_logit_weights = nn.Parameter(torch.randn(num_fine_quantizers, codebook_size, dim))
+        self.coarse_logit_weights = nn.Parameter(torch.randn(num_coarse_quantizers, codebook_size_with_eos, dim))
+        self.fine_logit_weights = nn.Parameter(torch.randn(num_fine_quantizers, codebook_size_with_eos, dim))
 
     def forward(
         self,
         coarse_token_ids,
         fine_token_ids,
         text = None,
-        text_embed = None
+        text_embed = None,
+        cond_drop_prob = None
     ):
-        device = coarse_token_ids.device
-
+        b, device = coarse_token_ids.shape[0], coarse_token_ids.device
         has_text = exists(text) or exists(text_embed)
         assert not (self.has_condition ^ has_text)
 
@@ -815,6 +888,12 @@ class FineTransformer(nn.Module):
                 text_embeds = self.embed_text(text, output_device = device)
                 text_mask = torch.any(text_embeds != 0, dim = -1)
 
+        cond_drop_prob = default(cond_drop_prob, self.cond_drop_prob)
+
+        if cond_drop_prob > 0:
+            keep_mask = prob_mask_like((b,), 1 - cond_drop_prob, device = device)
+            text_mask = rearrange(keep_mask, 'b -> b 1') & text_mask
+
         coarse_token_ids, fine_token_ids = map(lambda t: rearrange(t, 'b ... -> b (...)'), (coarse_token_ids, fine_token_ids))
 
         b, n = coarse_token_ids.shape
@@ -842,16 +921,25 @@ class FineTransformer(nn.Module):
 
         # get coarse logits
 
+        pred_coarse_seq_len = pred_coarse_tokens.shape[1]
+
+        padding = remainder_needed_until_multiple(pred_coarse_seq_len, self.num_coarse_quantizers)
+
+        if padding != 0:
+            pred_coarse_tokens = F.pad(pred_coarse_tokens, (0, 0, 0, padding), value = 0.)
+
         pred_coarse_tokens = rearrange(pred_coarse_tokens, 'b (n q) d -> b n q d', q = self.num_coarse_quantizers)
 
         coarse_logits = einsum('q c d, b n q d -> b n q c', self.coarse_logit_weights, pred_coarse_tokens)
 
         coarse_logits = rearrange(coarse_logits, 'b n q c -> b (n q) c')
 
+        coarse_logits = coarse_logits[:, :pred_coarse_seq_len]
+
         # get fine logits
 
-        n = pred_fine_tokens.shape[1]
-        nq = round_down_nearest_multiple(n, self.num_fine_quantizers)
+        pred_fine_seq_len = pred_fine_tokens.shape[1]
+        nq = round_down_nearest_multiple(pred_fine_seq_len, self.num_fine_quantizers)
 
         pred_fine_tokens_groupable, pred_fine_tokens_remainder = pred_fine_tokens[:, :nq], pred_fine_tokens[:, nq:]
 
@@ -911,6 +999,9 @@ class FineTransformerWrapper(nn.Module):
         coarse_token_ids = rearrange(coarse_token_ids, 'b ... -> b (...)')
         fine_token_ids = rearrange(fine_token_ids, 'b ... -> b (...)')
 
+        coarse_token_ids = append_eos_id(coarse_token_ids, self.transformer.eos_id)
+        fine_token_ids = append_eos_id(fine_token_ids, self.transformer.eos_id)
+
         if return_loss:
             coarse_labels, fine_labels = coarse_token_ids, fine_token_ids.clone()
             fine_token_ids = fine_token_ids[:, :-1]
@@ -945,7 +1036,8 @@ class CoarseTransformerWrapper(nn.Module):
         transformer: FineTransformer,
         soundstream: Optional[SoundStream]  = None,
         wav2vec: Optional[Union[FairseqVQWav2Vec, HubertWithKmeans]] = None,
-        num_coarse_quantize = 3
+        num_coarse_quantize = 3,
+        unique_consecutive = False
     ):
         super().__init__()
         self.soundstream = soundstream
@@ -953,6 +1045,8 @@ class CoarseTransformerWrapper(nn.Module):
 
         self.transformer = transformer
 
+        assert not unique_consecutive, 'not implemented yet'
+
         assert num_coarse_quantize > 0
         self.num_coarse_quantize = num_coarse_quantize
 
@@ -984,6 +1078,9 @@ class CoarseTransformerWrapper(nn.Module):
         coarse_token_ids = rearrange(coarse_token_ids, 'b ... -> b (...)')
         semantic_token_ids = rearrange(semantic_token_ids, 'b ... -> b (...)')
 
+        coarse_token_ids = append_eos_id(coarse_token_ids, self.transformer.coarse_eos_id)
+        semantic_token_ids = append_eos_id(semantic_token_ids, self.transformer.semantic_eos_id)
+
         if return_loss:
             semantic_labels, coarse_labels = semantic_token_ids, coarse_token_ids.clone()
             coarse_token_ids = coarse_token_ids[:, :-1]
